@@ -19,7 +19,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* $Id: get_audio.c,v 1.1.1.1 2002/06/21 12:35:24 myrina Exp $ */
+/* $Id: get_audio.c,v 1.1 2003/07/29 04:17:51 heroine Exp $ */
 
 
 #ifdef HAVE_CONFIG_H
@@ -49,12 +49,18 @@ char   *strchr(), *strrchr();
 # endif
 #endif
 
+
 #define         MAX_U_32_NUM            0xFFFFFFFF
 
 
 #include <math.h>
 #include <sys/stat.h>
 
+#ifdef __sun__
+/* woraround for SunOS 4.x, it has SEEK_* defined here */
+#include <unistd.h>
+#endif
+
 #include "lame.h"
 #include "main.h"
 #include "get_audio.h"
@@ -78,7 +84,7 @@ FILE   *musicin;
 int     lame_decode_initfile(const char *fullname,
                              mp3data_struct * const mp3data);
 #else
-int     lame_decode_initfile(FILE * const fd, mp3data_struct * const mp3data);
+int     lame_decode_initfile(FILE * fd, mp3data_struct * mp3data);
 #endif
 
 /* read mp3 file until mpglib returns one frame of PCM data */
@@ -98,10 +104,10 @@ int     lame_decode_ogg_fromfile( lame_global_flags*  gfc,
 
 static int read_samples_pcm(FILE * musicin, int sample_buffer[2304],
                             int frame_size, int samples_to_read);
-static int read_samples_mp3(lame_global_flags * gfp, FILE * musicin,
-                            short int mpg123pcm[2][1152], int num_chan);
-static int read_samples_ogg(lame_global_flags * gfp, FILE * musicin,
+static int read_samples_mp3(lame_global_flags * const gfp, FILE * const musicin,
                             short int mpg123pcm[2][1152], int num_chan);
+static int read_samples_ogg(lame_global_flags * const gfp, FILE * const musicin,
+                            short int mpg123pcm[2][1152], const int num_chan);
 void    CloseSndFile(sound_file_format input, FILE * musicin);
 FILE   *OpenSndFile(lame_global_flags * gfp, char *);
 
@@ -1615,12 +1621,18 @@ lame_decode_fromfile(FILE * fd, short pcm_l[], short pcm_r[],
         if (len == 0) {
 	    /* we are done reading the file, but check for buffered data */
 	    ret = lame_decode1_headers(buf, len, pcm_l, pcm_r, mp3data);
-	    if (ret<=0) return -1;  // done with file
+	    if (ret<=0) {
+                lame_decode_exit(); // release mp3decoder memory
+                return -1;  // done with file
+            }
 	    break;
 	}
 
         ret = lame_decode1_headers(buf, len, pcm_l, pcm_r, mp3data);
-        if (ret == -1) return -1;
+        if (ret == -1) {
+            lame_decode_exit();  // release mp3decoder memory
+            return -1;
+        }
 	if (ret >0) break;
     }
     return ret;
