@@ -20,7 +20,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* $Id: main.c,v 1.1.1.1 2002/06/21 12:35:24 myrina Exp $ */
+/* $Id: main.c,v 1.1 2003/07/29 04:17:51 heroine Exp $ */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -66,6 +66,7 @@ char   *strchr(), *strrchr();
 #include "get_audio.h"
 #include "portableio.h"
 #include "timestatus.h"
+#include "VbrTag.h"
 
 /* PLL 14/04/2000 */
 #if macintosh
@@ -503,7 +504,7 @@ void parse_nogap_filenames(int nogapout, char *inPath, char *outPath, char *outd
 
     strcpy(outPath,outdir);
     if (!nogapout) 	{
-        strncpy(outPath, inPath, MAX_NAME_SIZE - 4);
+        strncpy(outPath, inPath, PATH_MAX + 1 - 4);
         n=strlen(outPath);
         /* nuke old extension, if one  */
         if (outPath[n-3] == 'w' 
@@ -522,7 +523,7 @@ void parse_nogap_filenames(int nogapout, char *inPath, char *outPath, char *outd
         }
     } else 	{
         slasher = inPath;
-        slasher += MAX_NAME_SIZE-4;
+        slasher += PATH_MAX + 1 - 4;
         
         /* backseek to last dir delemiter */
         while (*slasher != '/' && *slasher != '\\' && slasher != inPath
@@ -553,7 +554,7 @@ void parse_nogap_filenames(int nogapout, char *inPath, char *outPath, char *outd
         strcat(outPath, "/");
 #endif
         
-        strncat(outPath, slasher, MAX_NAME_SIZE-4);
+        strncat(outPath, slasher, PATH_MAX + 1 - 4);
         n=strlen(outPath);
         /* nuke old extension  */
         if (outPath[n-3] == 'w' 
@@ -584,9 +585,9 @@ main(int argc, char **argv)
 {
     int     ret;
     lame_global_flags *gf;
-    char    outPath[MAX_NAME_SIZE];
-    char    nogapdir[MAX_NAME_SIZE];
-    char    inPath[MAX_NAME_SIZE];
+    char    outPath[PATH_MAX + 1];
+    char    nogapdir[PATH_MAX + 1];
+    char    inPath[PATH_MAX + 1];
 
     /* support for "nogap" encoding of up to 200 .wav files */
 #define MAX_NOGAP 200
@@ -622,7 +623,7 @@ main(int argc, char **argv)
 #endif
 
     for (i = 0; i < max_nogap; ++i) {
-        nogap_inPath[i] = malloc(MAX_NAME_SIZE);
+        nogap_inPath[i] = malloc(PATH_MAX + 1);
     }
 
     memset(inPath, 0, sizeof(inPath));
@@ -653,7 +654,7 @@ main(int argc, char **argv)
         update_interval = 2.;
 
     if (outPath[0] != '\0' && max_nogap>0) {
-        strncpy(nogapdir,outPath,MAX_NAME_SIZE);  
+        strncpy(nogapdir, outPath, PATH_MAX + 1);  
         nogapout = 1;
     }
     
