@@ -1,4 +1,4 @@
-/* $Id: mp3x.c,v 1.1.1.1 2002/06/21 12:35:24 myrina Exp $ */
+/* $Id: mp3x.c,v 1.1 2003/07/29 04:17:51 heroine Exp $ */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -36,8 +36,8 @@ int main(int argc, char **argv)
 {
   char mp3buffer[LAME_MAXMP3BUFFER];
   lame_global_flags *gf;  
-  char outPath[MAX_NAME_SIZE];
-  char inPath[MAX_NAME_SIZE];
+  char outPath[PATH_MAX + 1];
+  char inPath[PATH_MAX + 1];
   int ret;
 
   gf=lame_init();
