@@ -18,7 +18,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: bitstream.c,v 1.1.1.1 2002/06/21 12:35:24 myrina Exp $
+ * $Id: bitstream.c,v 1.1 2003/07/29 04:17:51 heroine Exp $
  */
 
 
@@ -79,9 +79,8 @@ void putheader_bits(lame_internal_flags *gfc,int w_ptr)
 
 /*write j bits into the bit stream */
 inline static void
-putbits2(lame_global_flags *gfp, int val, int j)
+putbits2(lame_internal_flags *gfc, int val, int j)
 {
-    lame_internal_flags *gfc=gfp->internal_flags;
     Bit_stream_struc *bs;
     bs = &gfc->bs;
 
@@ -115,9 +114,8 @@ putbits2(lame_global_flags *gfp, int val, int j)
 
 /*write j bits into the bit stream, ignoring frame headers */
 inline static void
-putbits_noheaders(lame_global_flags *gfp, int val, int j)
+putbits_noheaders(lame_internal_flags *gfc, int val, int j)
 {
-    lame_internal_flags *gfc=gfp->internal_flags;
     Bit_stream_struc *bs;
     bs = &gfc->bs;
 
@@ -154,26 +152,25 @@ putbits_noheaders(lame_global_flags *gfp, int val, int j)
 */
 
 inline static void
-drain_into_ancillary(lame_global_flags *gfp,int remainingBits)
+drain_into_ancillary(lame_internal_flags *gfc, int remainingBits)
 {
-    lame_internal_flags *gfc=gfp->internal_flags;
     int i;
     assert(remainingBits >= 0);
 
     if (remainingBits >= 8) {
-      putbits2(gfp,0x4c,8);
+      putbits2(gfc,0x4c,8);
       remainingBits -= 8;
     }
     if (remainingBits >= 8) {
-      putbits2(gfp,0x41,8);
+      putbits2(gfc,0x41,8);
       remainingBits -= 8;
     }
     if (remainingBits >= 8) {
-      putbits2(gfp,0x4d,8);
+      putbits2(gfc,0x4d,8);
       remainingBits -= 8;
     }
     if (remainingBits >= 8) {
-      putbits2(gfp,0x45,8);
+      putbits2(gfc,0x45,8);
       remainingBits -= 8;
     }
       
@@ -182,12 +179,12 @@ drain_into_ancillary(lame_global_flags *gfp,int remainingBits)
       if (remainingBits >= 32) 
 	for (i=0; i<(int)strlen(version) && remainingBits >=8 ; ++i) {
 	  remainingBits -= 8;
-	  putbits2(gfp,version[i],8);
+	  putbits2(gfc,version[i],8);
 	}
     }
 
     for (; remainingBits >= 1; remainingBits -= 1 ) {
-        putbits2 ( gfp, gfc->ancillary_flag, 1 );
+        putbits2(gfc, gfc->ancillary_flag, 1 );
         gfc->ancillary_flag ^= 1;
     }
 
@@ -294,14 +291,14 @@ encodeSideInfo2(lame_global_flags *gfp,int bitsPerFrame)
 
 	for (gr = 0; gr < 2; gr++) {
 	    for (ch = 0; ch < gfc->channels_out; ch++) {
-		gr_info *gi = &l3_side->gr[gr].ch[ch].tt;
+		gr_info *gi = &l3_side->tt[gr][ch];
 		writeheader(gfc,gi->part2_3_length,       12);
 		writeheader(gfc,gi->big_values / 2,        9);
 		writeheader(gfc,gi->global_gain,           8);
 		writeheader(gfc,gi->scalefac_compress,     4);
-		writeheader(gfc,gi->window_switching_flag, 1);
 
-		if (gi->window_switching_flag) {
+		if (gi->block_type != NORM_TYPE) {
+		    writeheader(gfc, 1, 1); // window_switching_flag
 		    writeheader(gfc,gi->block_type,       2);
 		    writeheader(gfc,gi->mixed_block_flag, 1);
 
@@ -316,7 +313,7 @@ encodeSideInfo2(lame_global_flags *gfp,int bitsPerFrame)
 		    writeheader(gfc,gi->subblock_gain[1], 3);
 		    writeheader(gfc,gi->subblock_gain[2], 3);
 		} else {
-		    assert(gi->block_type == NORM_TYPE);
+		    writeheader(gfc, 0, 1); // window_switching_flag
 		    if (gi->table_select[0] == 14)
 			gi->table_select[0] = 16;
 		    writeheader(gfc,gi->table_select[0], 5);
@@ -345,14 +342,14 @@ encodeSideInfo2(lame_global_flags *gfp,int bitsPerFrame)
 
 	gr = 0;
 	for (ch = 0; ch < gfc->channels_out; ch++) {
-	    gr_info *gi = &l3_side->gr[gr].ch[ch].tt;
+	    gr_info *gi = &l3_side->tt[gr][ch];
 	    writeheader(gfc,gi->part2_3_length,       12);
 	    writeheader(gfc,gi->big_values / 2,        9);
 	    writeheader(gfc,gi->global_gain,           8);
 	    writeheader(gfc,gi->scalefac_compress,     9);
-	    writeheader(gfc,gi->window_switching_flag, 1);
 
-	    if (gi->window_switching_flag) {
+	    if (gi->block_type != NORM_TYPE) {
+		writeheader(gfc, 1, 1); // window_switching_flag
 		writeheader(gfc,gi->block_type,       2);
 		writeheader(gfc,gi->mixed_block_flag, 1);
 
@@ -367,6 +364,7 @@ encodeSideInfo2(lame_global_flags *gfp,int bitsPerFrame)
 		writeheader(gfc,gi->subblock_gain[1], 3);
 		writeheader(gfc,gi->subblock_gain[2], 3);
 	    } else {
+		writeheader(gfc, 0, 1); // window_switching_flag
 		if (gi->table_select[0] == 14)
 		    gi->table_select[0] = 16;
 		writeheader(gfc,gi->table_select[0], 5);
@@ -411,11 +409,8 @@ encodeSideInfo2(lame_global_flags *gfp,int bitsPerFrame)
 
 
 inline static int
-huffman_coder_count1(lame_global_flags *gfp,int *ix, gr_info *gi)
+huffman_coder_count1(lame_internal_flags *gfc, gr_info *gi)
 {
-#ifdef DEBUG
-    lame_internal_flags *gfc = gfp->internal_flags;
-#endif
     /* Write count1 area */
     const struct huffcodetab *h = &ht[gi->count1table_select + 32];
     int i,bits=0;
@@ -423,10 +418,10 @@ huffman_coder_count1(lame_global_flags *gfp,int *ix, gr_info *gi)
     int gegebo = gfc->bs.totbit;
 #endif
 
-    ix += gi->big_values;
+    int *ix = &gi->l3_enc[gi->big_values];
+    FLOAT8 *xr = &gi->xr[gi->big_values];
     assert(gi->count1table_select < 2);
 
-
     for (i = (gi->count1 - gi->big_values) / 4; i > 0; --i) {
 	int huffbits = 0;
 	int p = 0, v;
@@ -434,40 +429,41 @@ huffman_coder_count1(lame_global_flags *gfp,int *ix, gr_info *gi)
 	v = ix[0];
 	if (v) {
 	    p += 8;
-	    if (v < 0)
+	    if (xr[0] < 0)
 		huffbits++;
-	    assert(-1 <= v && v <= 1);
+	    assert(v <= 1u);
 	}
 
 	v = ix[1];
 	if (v) {
 	    p += 4;
 	    huffbits *= 2;
-	    if (v < 0)
+	    if (xr[1] < 0)
 		huffbits++;
-	    assert(-1 <= v && v <= 1);
+	    assert(v <= 1u);
 	}
 
 	v = ix[2];
 	if (v) {
 	    p += 2;
 	    huffbits *= 2;
-	    if (v < 0)
+	    if (xr[2] < 0)
 		huffbits++;
-	    assert(-1 <= v && v <= 1);
+	    assert(v <= 1u);
 	}
 
 	v = ix[3];
 	if (v) {
 	    p++;
 	    huffbits *= 2;
-	    if (v < 0)
+	    if (xr[3] < 0)
 		huffbits++;
-	    assert(-1 <= v && v <= 1);
+	    assert(v <= 1u);
 	}
 
 	ix += 4;
-	putbits2(gfp,huffbits + h->table[p], h->hlen[p]);
+	xr += 4;
+	putbits2(gfc, huffbits + h->table[p], h->hlen[p]);
 	bits += h->hlen[p];
     }
 #ifdef DEBUG
@@ -483,32 +479,27 @@ huffman_coder_count1(lame_global_flags *gfp,int *ix, gr_info *gi)
   */
 
 inline static int
-HuffmanCode ( lame_global_flags* const gfp, const int table_select, int x1, int x2 )
+HuffmanCode( lame_internal_flags* const gfc, const int table_select,
+	     int index, gr_info *gi)
 {
     const struct huffcodetab* h = ht + table_select;
-    int  code    = 0;
     int  cbits   = 0;
     int  xbits   = 0;
-    int  sgn_x1  = 0;
-    int  sgn_x2  = 0;
     int  linbits = h->xlen;
     int  xlen    = h->xlen;
-    int  ext; 
+    int  ext = 0;
+    int x1 = gi->l3_enc[index];
+    int x2 = gi->l3_enc[index+1];
 
     assert ( table_select > 0 );
 
-    if (x1 < 0) {
-	sgn_x1++;
-	x1 = -x1;
-    }
+    if (x1 != 0) {
+	if (gi->xr[index] < 0)
+	    ext++;
 
-    if (x2 < 0) {
-	sgn_x2++;
-	x2 = -x2;
+	cbits--;
     }
 
-    ext     = sgn_x1;
-
     if (table_select > 15) {
 	/* use ESC-words */
 	if (x1 > 14) {
@@ -530,13 +521,10 @@ HuffmanCode ( lame_global_flags* const gfp, const int table_select, int x1, int
 	xlen = 16;
     }
 
-    if (x1 != 0) {
-	cbits--;
-    }
-
     if (x2 != 0) {
 	ext <<= 1;
-	ext  |= sgn_x2;
+	if (gi->xr[index+1] < 0)
+	    ext++;
 	cbits--;
     }
 
@@ -546,20 +534,19 @@ HuffmanCode ( lame_global_flags* const gfp, const int table_select, int x1, int
 
     x1 = x1 * xlen + x2;
 
-    code   = h->table [x1];
     cbits += h->hlen  [x1];
 
     assert ( cbits <= MAX_LENGTH );
     assert ( xbits <= MAX_LENGTH );
 
-    putbits2 ( gfp, code, cbits );
-    putbits2 ( gfp, ext,  xbits );
+    putbits2(gfc, h->table [x1], cbits );
+    putbits2(gfc, ext,  xbits );
     
     return cbits + xbits;
 }
 
 static int
-Huffmancodebits(lame_global_flags *gfp, int tableindex, int start, int end, int *ix)
+Huffmancodebits(lame_internal_flags *gfc, int tableindex, int start, int end, gr_info *gi)
 {
     int i,bits;
 
@@ -567,9 +554,9 @@ Huffmancodebits(lame_global_flags *gfp, int tableindex, int start, int end, int
     if (!tableindex) return 0;
 
     bits=0;
-    for (i = start; i < end; i += 2) {
-      bits +=HuffmanCode(gfp,tableindex, ix[i], ix[i + 1]);
-    }
+    for (i = start; i < end; i += 2)
+      bits +=HuffmanCode(gfc, tableindex, i, gi);
+
     return bits;
 }
 
@@ -581,27 +568,25 @@ Huffmancodebits(lame_global_flags *gfp, int tableindex, int start, int end, int
   information on pages 26 and 27.
   */
 static int
-ShortHuffmancodebits(lame_global_flags *gfp,int *ix, gr_info *gi)
+ShortHuffmancodebits(lame_internal_flags *gfc, gr_info *gi)
 {
-    lame_internal_flags *gfc=gfp->internal_flags;
     int bits;
     int region1Start;
     
     region1Start = 3*gfc->scalefac_band.s[3];
-    if (region1Start > gi->big_values) 	
-        region1Start = gi->big_values;
+    if (region1Start > gi->big_values)
+	region1Start = gi->big_values;
 
     /* short blocks do not have a region2 */
-    bits  = Huffmancodebits(gfp,gi->table_select[0], 0, region1Start, ix);
-    bits += Huffmancodebits(gfp,gi->table_select[1], region1Start, gi->big_values, ix);
+    bits  = Huffmancodebits(gfc, gi->table_select[0], 0, region1Start, gi);
+    bits += Huffmancodebits(gfc, gi->table_select[1], region1Start, gi->big_values, gi);
     return bits;
 }
 
 static int
-LongHuffmancodebits(lame_global_flags *gfp,int *ix, gr_info *gi)
+LongHuffmancodebits(lame_internal_flags *gfc, gr_info *gi)
 {
-    lame_internal_flags *gfc=gfp->internal_flags;
-    int i, bigvalues,bits=0;
+    int i, bigvalues, bits;
     int region1Start, region2Start;
 
     bigvalues = gi->big_values;
@@ -620,16 +605,14 @@ LongHuffmancodebits(lame_global_flags *gfp,int *ix, gr_info *gi)
     if (region2Start > bigvalues)
 	region2Start = bigvalues;
 
-    bits +=Huffmancodebits(gfp,gi->table_select[0], 0, region1Start, ix);
-    bits +=Huffmancodebits(gfp,gi->table_select[1], region1Start, region2Start, ix);
-    bits +=Huffmancodebits(gfp,gi->table_select[2], region2Start, bigvalues, ix);
+    bits  =Huffmancodebits(gfc, gi->table_select[0], 0, region1Start, gi);
+    bits +=Huffmancodebits(gfc, gi->table_select[1], region1Start, region2Start, gi);
+    bits +=Huffmancodebits(gfc, gi->table_select[2], region2Start, bigvalues, gi);
     return bits;
 }
 
 inline static int
-writeMainData ( lame_global_flags * const gfp,
-        int              l3_enc   [2] [2] [576],
-        III_scalefac_t   scalefac [2] [2] )
+writeMainData ( lame_global_flags * const gfp)
 {
     int gr, ch, sfb,data_bits,scale_bits,tot_bits=0;
     lame_internal_flags *gfc=gfp->internal_flags;
@@ -640,7 +623,7 @@ writeMainData ( lame_global_flags * const gfp,
 	/* MPEG 1 */
 	for (gr = 0; gr < 2; gr++) {
 	    for (ch = 0; ch < gfc->channels_out; ch++) {
-		gr_info *gi = &l3_side->gr[gr].ch[ch].tt;
+		gr_info *gi = &l3_side->tt[gr][ch];
 		int slen1 = slen1_tab[gi->scalefac_compress];
 		int slen2 = slen2_tab[gi->scalefac_compress];
 		data_bits=0;
@@ -653,16 +636,16 @@ writeMainData ( lame_global_flags * const gfp,
 		    for (sfb = 0; sfb < SBPSY_s; sfb++) {
 			int slen = sfb < 6 ? slen1 : slen2;
 
-			assert(scalefac[gr][ch].s[sfb][0]>=0);
-			assert(scalefac[gr][ch].s[sfb][1]>=0);
-			assert(scalefac[gr][ch].s[sfb][2]>=0);
+			assert(gi->scalefac.s[sfb][0]>=0);
+			assert(gi->scalefac.s[sfb][1]>=0);
+			assert(gi->scalefac.s[sfb][2]>=0);
 
-			putbits2(gfp,scalefac[gr][ch].s[sfb][0], slen);
-			putbits2(gfp,scalefac[gr][ch].s[sfb][1], slen);
-			putbits2(gfp,scalefac[gr][ch].s[sfb][2], slen);
+			putbits2(gfc, gi->scalefac.s[sfb][0], slen);
+			putbits2(gfc, gi->scalefac.s[sfb][1], slen);
+			putbits2(gfc, gi->scalefac.s[sfb][2], slen);
 			scale_bits += 3*slen;
 		    }
-		    data_bits += ShortHuffmancodebits(gfp,l3_enc[gr][ch], gi);
+		    data_bits += ShortHuffmancodebits(gfc, gi);
 		} else {
 		    int i;
 		    for (i = 0; i < sizeof(scfsi_band) / sizeof(int) - 1;
@@ -673,15 +656,15 @@ writeMainData ( lame_global_flags * const gfp,
 			for (sfb = scfsi_band[i]; sfb < scfsi_band[i + 1];
 			     sfb++) {
 
-			    assert(scalefac[gr][ch].l[sfb]>=0);
-			    putbits2(gfp,scalefac[gr][ch].l[sfb],
+			    assert(gi->scalefac.l[sfb]>=0);
+			    putbits2(gfc, gi->scalefac.l[sfb],
 				    sfb < 11 ? slen1 : slen2);
 			    scale_bits += sfb < 11 ? slen1 : slen2;
 			}
 		    }
-		    data_bits +=LongHuffmancodebits(gfp,l3_enc[gr][ch], gi);
+		    data_bits +=LongHuffmancodebits(gfc, gi);
 		}
-		data_bits +=huffman_coder_count1(gfp,l3_enc[gr][ch], gi);
+		data_bits +=huffman_coder_count1(gfc, gi);
 #ifdef DEBUG
 		DEBUGF(gfc,"<%ld> ", gfc->bs.totbit-hogege);
 #endif
@@ -696,7 +679,7 @@ writeMainData ( lame_global_flags * const gfp,
 	/* MPEG 2 */
 	gr = 0;
 	for (ch = 0; ch < gfc->channels_out; ch++) {
-	    gr_info *gi = &l3_side->gr[gr].ch[ch].tt;
+	    gr_info *gi = &l3_side->tt[gr][ch];
 	    int i, sfb_partition;
 	    assert(gi->sfb_partition_table);
 	    data_bits = 0;
@@ -704,30 +687,31 @@ writeMainData ( lame_global_flags * const gfp,
 
 	    sfb = 0;
 	    sfb_partition = 0;
+
 	    if (gi->block_type == SHORT_TYPE) {
 		for (; sfb_partition < 4; sfb_partition++) {
 		    int sfbs = gi->sfb_partition_table[sfb_partition] / 3;
 		    int slen = gi->slen[sfb_partition];
 		    for (i = 0; i < sfbs; i++, sfb++) {
-			putbits2(gfp,Max(scalefac[gr][ch].s[sfb][0], 0U), slen);
-			putbits2(gfp,Max(scalefac[gr][ch].s[sfb][1], 0U), slen);
-			putbits2(gfp,Max(scalefac[gr][ch].s[sfb][2], 0U), slen);
+			putbits2(gfc, Max(gi->scalefac.s[sfb][0], 0U), slen);
+			putbits2(gfc, Max(gi->scalefac.s[sfb][1], 0U), slen);
+			putbits2(gfc, Max(gi->scalefac.s[sfb][2], 0U), slen);
 			scale_bits += 3*slen;
 		    }
 		}
-		data_bits += ShortHuffmancodebits(gfp,l3_enc[gr][ch], gi);
+		data_bits += ShortHuffmancodebits(gfc, gi);
 	    } else {
 		for (; sfb_partition < 4; sfb_partition++) {
 		    int sfbs = gi->sfb_partition_table[sfb_partition];
 		    int slen = gi->slen[sfb_partition];
 		    for (i = 0; i < sfbs; i++, sfb++) {
-			putbits2(gfp,Max(scalefac[gr][ch].l[sfb], 0U), slen);
+			putbits2(gfc, Max(gi->scalefac.l[sfb], 0U), slen);
 			scale_bits += slen;
 		    }
 		}
-		data_bits +=LongHuffmancodebits(gfp,l3_enc[gr][ch], gi);
+		data_bits +=LongHuffmancodebits(gfc, gi);
 	    }
-	    data_bits +=huffman_coder_count1(gfp,l3_enc[gr][ch], gi);
+	    data_bits +=huffman_coder_count1(gfc, gi);
 
 	    /* does bitcount in quantize.c agree with actual bit count?*/
 	    assert(data_bits==gi->part2_3_length-gi->part2_length);
@@ -831,7 +815,7 @@ flush_bitstream(lame_global_flags *gfp)
 
 
   if ((flushbits = compute_flushbits(gfp,&nbytes)) < 0) return;  
-  drain_into_ancillary(gfp,flushbits);
+  drain_into_ancillary(gfc, flushbits);
 
   /* check that the 100% of the last frame has been written to bitstream */
   getframebits(gfp,&bitsPerFrame,&mean_bits);
@@ -852,7 +836,7 @@ void  add_dummy_byte ( lame_global_flags* const gfp, unsigned char val )
   lame_internal_flags *gfc = gfp->internal_flags;
   int i;
 
-  putbits_noheaders(gfp,val,8);   
+  putbits_noheaders(gfc, val, 8);   
 
   for (i=0 ; i< MAX_HEADER_BUF ; ++i) 
     gfc->header[i].write_timing += 8;
@@ -870,21 +854,19 @@ void  add_dummy_byte ( lame_global_flags* const gfp, unsigned char val )
   in the IS).
   */
 int
-format_bitstream(lame_global_flags *gfp, int bitsPerFrame,
-      int              l3_enc[2][2][576],
-  	III_scalefac_t   scalefac[2][2] )
+format_bitstream(lame_global_flags *gfp, int bitsPerFrame)
 {
     lame_internal_flags *gfc=gfp->internal_flags;
     int bits,nbytes;
     III_side_info_t *l3_side;
     l3_side = &gfc->l3_side;
 
-    drain_into_ancillary(gfp,l3_side->resvDrain_pre);
+    drain_into_ancillary(gfc, l3_side->resvDrain_pre);
 
     encodeSideInfo2(gfp,bitsPerFrame);
     bits = 8*gfc->sideinfo_len;
-    bits+=writeMainData(gfp,l3_enc,scalefac);
-    drain_into_ancillary(gfp,l3_side->resvDrain_post);
+    bits+=writeMainData(gfp);
+    drain_into_ancillary(gfc, l3_side->resvDrain_post);
     bits += l3_side->resvDrain_post;
 
     l3_side->main_data_begin += (bitsPerFrame-bits)/8;
