@@ -20,7 +20,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* $Id: lame.c,v 1.1.1.1 2002/06/21 12:35:24 myrina Exp $ */
+/* $Id: lame.c,v 1.1 2003/07/29 04:17:51 heroine Exp $ */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -48,6 +48,13 @@
 #include <dmalloc.h>
 #endif
 
+#ifdef __sun__
+/* woraround for SunOS 4.x, it has SEEK_* defined here */
+#include <unistd.h>
+#endif
+
+
+#define DEFAULT_QUALITY 2
 
 static void
 lame_init_params_ppflt_lowpass(FLOAT8 amp_lowpass[32], FLOAT lowpass1,
@@ -467,7 +474,7 @@ lame_init_qval(lame_global_flags * gfp)
         gfc->quantization = 1;
         gfc->noise_shaping = 1; /* 2=usually lowers quality */
         gfc->substep_shaping = 2;
-        gfc->noise_shaping_amp = 3;
+        gfc->noise_shaping_amp = 2;
         gfc->noise_shaping_stop = 1;
         gfc->use_best_huffman = 1; /* 2 not yet coded */
     }
@@ -607,6 +614,7 @@ lame_init_params(lame_global_flags * const gfp)
 
     if (gfp->VBR != vbr_off) {
         gfp->free_format = 0; /* VBR can't be mixed with free format */
+        gfp->padding_type = PAD_NO;    
     }
 
     if (gfp->VBR == vbr_off && gfp->brate == 0) {
@@ -1016,7 +1024,7 @@ lame_init_params(lame_global_flags * const gfp)
     case vbr_mtrh:
 
         if (gfp->ATHtype < 0) gfp->ATHtype = 4;
-        if (gfp->quality < 0) gfp->quality = 2;
+        if (gfp->quality < 0) gfp->quality = DEFAULT_QUALITY;
         if (gfp->quality > 7) {
             gfp->quality = 7;     // needs psymodel
             ERRORF( gfc, "VBR needs a psymodel, switching to quality level 7\n");
@@ -1027,43 +1035,64 @@ lame_init_params(lame_global_flags * const gfp)
         if (gfp->cwlimit <= 0) gfp->cwlimit = 0.42 * gfp->out_samplerate;
         gfc->PSY->tonalityPatch = 1;
 
-        if ( gfp->experimentalX <= 4 && gfp->experimentalX >= 0 )
-        {   /* map experimentalX settings to internal secltions */
-            static char const map[] = {2,1,0,3,6};
-            gfc->VBR->quality = map[gfp->experimentalX];
+        switch ( gfp->experimentalX ) {
+        default:
+        case 0: {
+                static const float dbQ[10]={-2.,-1.0,-.66,-.33,0.,0.33,.66,1.0,1.33,1.66};
+                gfc->VBR->quality = 0;
+                gfc->VBR->mask_adjust = dbQ[gfp->VBR_q];
+                gfc->VBR->smooth = 1;
+            } 
+            break;        
+        case 1: {
+                static float const dbQ[10] = { -2., -1.4, -.7, 0, .7, 1.5, 2.3, 3.1, 4., 5 };
+                gfc->VBR->quality = 1;
+                gfc->VBR->mask_adjust = dbQ[gfp->VBR_q];
+                gfc->VBR->smooth = 0;    
+            } 
+            break;        
+        case 2: {
+                static float const dbQ[10] = { -1., -.6, -.3, 0, 1, 2, 3, 4, 5, 6};
+                gfc->VBR->quality = 2;
+                gfc->VBR->mask_adjust = dbQ[gfp->VBR_q];
+                gfc->VBR->smooth = 0;    
+                gfc->PSY->tonalityPatch = 0;
+            } 
+            break;        
+        case 3: {
+                static const float dbQ[10]={-2.,-1.0,-.66,-.33,0.,0.33,.66,1.0,1.33,1.66};
+                gfc->VBR->quality = 3;
+                gfc->VBR->mask_adjust = dbQ[gfp->VBR_q];
+                gfc->VBR->smooth = 1;
+            } 
+            break;        
+        case 4: {
+                static float const dbQ[10] = { -6,-4.75,-3.5,-2.25,-1,.25,1.5,2.75,4,5.25 };
+                gfc->VBR->quality = 4;
+                gfc->VBR->mask_adjust = dbQ[gfp->VBR_q];
+                gfc->VBR->smooth = 1;   // not finally
+            }
+            break;        
+        case 5: {
+                static const float dbQ[10]={-2.,-1.0,-.66,-.33,0.,0.33,.66,1.0,1.33,1.66};
+                gfc->VBR->quality = 0;
+                gfc->VBR->mask_adjust = dbQ[gfp->VBR_q];
+                gfc->VBR->smooth = 2;
+            } 
+            break;        
+        case 9: {
+                static float const dbQ[10] = { -6,-4.75,-3.5,-2.25,-1,.25,1.5,2.75,4,5.25 };
+                gfc->VBR->quality = 4;
+                gfc->VBR->mask_adjust = dbQ[gfp->VBR_q];
+                gfc->VBR->smooth = 0;   // not finally
+            }
+            break;        
         }
-        else    /* defaulting to */
-        {
-            gfc->VBR->quality = 2;
-        }   
-        if ( gfc->VBR->quality > 5 ) {
-            static float const dbQ[10] = { -6,-4.75,-3.5,-2.25,-1,.25,1.5,2.75,4,5.25 };
-            gfc->VBR->mask_adjust = dbQ[gfp->VBR_q];
-            gfc->VBR->smooth = 1;   // not finally
-        }
-        else {
-            static const float dbQ[10]={-2.,-1.0,-.66,-.33,0.,0.33,.66,1.0,1.33,1.66};
-            gfc->VBR->mask_adjust = dbQ[gfp->VBR_q];
-            gfc->VBR->smooth = 1;
-        }
-        if ( gfc->VBR->quality == 1 ) {
-            static float const dbQ[10] = { -2., -1.4, -.7, 0, .7, 1.5, 2.3, 3.1, 4., 5 };
-            gfc->VBR->mask_adjust = dbQ[gfp->VBR_q];
-            gfc->VBR->smooth = 0;    
-        }
-        if ( gfc->VBR->quality == 0 ) {
-            static float const dbQ[10] = { -1., -.6, -.3, 0, 1, 2, 3, 4, 5, 6};
-            gfc->VBR->mask_adjust = dbQ[gfp->VBR_q];
-            gfc->VBR->smooth = 0;    
-            gfc->PSY->tonalityPatch = 0;
-        }
-        
-        gfc->VBR->bitpressure = 1;
         
         if (gfp->experimentalY)
             gfc->sfb21_extra = 0;
         else
-            gfc->sfb21_extra = (gfp->out_samplerate > 44000);
+            gfc->sfb21_extra = (gfp->out_samplerate > 36000);
         
         if ( gfp->athaa_type < 0 )
             gfc->ATH->use_adjust = 3;
@@ -1086,7 +1115,6 @@ lame_init_params(lame_global_flags * const gfp)
                 gfc->VBR->mask_adjust = dbQ[gfp->VBR_q]; 
             }
         }
-        gfc->VBR->bitpressure = 1;
         
         /*  use Gabriel's adaptative ATH shape for VBR by default
          */
@@ -1121,7 +1149,7 @@ lame_init_params(lame_global_flags * const gfp)
         /*  default quality setting is 2
          */
         if (gfp->quality < 0)
-            gfp->quality = 2;
+            gfp->quality = DEFAULT_QUALITY;
 
         break;
 
@@ -1146,7 +1174,7 @@ lame_init_params(lame_global_flags * const gfp)
         /*  default quality setting for CBR/ABR is 2
          */
         if (gfp->quality < 0)
-            gfp->quality = 2;
+            gfp->quality = DEFAULT_QUALITY;
 
         break;
     }
@@ -1182,11 +1210,6 @@ lame_init_params(lame_global_flags * const gfp)
     
     if (gfp->useTemporal < 0 ) gfp->useTemporal = 1;  // on by default
 
-
-    if ( gfp->experimentalY )
-        MSGF(gfc,"\n *** WARNING *** the meaning of the experimental -Y has changed!\n"
-                   "                 now it tells LAME to ignore sfb21 noise shaping (VBR)\n\n");
-
     if ( gfp->preset_expopts && gfc->presetTune.use < 1 )
         MSGF(gfc,"\n*** WARNING ***\n\n"
 		         "Specialized tunings for the preset you are using have been deactivated.\n"
@@ -1454,14 +1477,19 @@ lame_encode_frame(lame_global_flags * gfp,
  * mp3buffer_size in bytes = 1.25*num_samples + 7200
  *
  * return code = number of bytes output in mp3buffer.  can be 0
+ *
+ * NOTE: this routine uses LAME's internal PCM data representation,
+ * 'sample_t'.  It should not be used by any application.  
+ * applications should use lame_encode_buffer(), 
+ *                         lame_encode_buffer_float()
+ *                         lame_encode_buffer_int()
+ * etc... depending on what type of data they are working with.  
 */
 int
 lame_encode_buffer_sample_t(lame_global_flags * gfp,
                    sample_t buffer_l[],
                    sample_t buffer_r[],
-                   int nsamples, 
-				   unsigned char *mp3buf, 
-				   const int mp3buf_size)
+                   int nsamples, unsigned char *mp3buf, const int mp3buf_size)
 {
     lame_internal_flags *gfc = gfp->internal_flags;
     int     mp3size = 0, ret, i, ch, mf_needed;
@@ -1486,6 +1514,42 @@ lame_encode_buffer_sample_t(lame_global_flags * gfp,
     in_buffer[1]=buffer_r;  
 
 
+    /* Apply user defined re-scaling */
+
+    /* user selected scaling of the samples */
+    if (gfp->scale != 0 && gfp->scale != 1.0) {
+	for (i=0 ; i<nsamples; ++i) {
+	    in_buffer[0][i] *= gfp->scale;
+	    if (gfc->channels_out == 2)
+		in_buffer[1][i] *= gfp->scale;
+	    }
+    }
+
+    /* user selected scaling of the channel 0 (left) samples */
+    if (gfp->scale_left != 0 && gfp->scale_left != 1.0) {
+	for (i=0 ; i<nsamples; ++i) {
+	    in_buffer[0][i] *= gfp->scale_left;
+	    }
+    }
+
+    /* user selected scaling of the channel 1 (right) samples */
+	if (gfp->scale_right != 0 && gfp->scale_right != 1.0) {
+	    for (i=0 ; i<nsamples; ++i) {
+		in_buffer[1][i] *= gfp->scale_right;
+	    }
+	}
+
+    /* Downsample to Mono if 2 channels in and 1 channel out */
+	if (gfp->num_channels == 2 && gfc->channels_out == 1) {
+		for (i=0; i<nsamples; ++i) {
+			in_buffer[0][i] =
+				0.5 * ((FLOAT8) in_buffer[0][i] + in_buffer[1][i]);
+			in_buffer[1][i] = 0.0;
+		}
+	}
+
+
+
     /* some sanity checks */
 #if ENCDELAY < MDCTDELAY
 # error ENCDELAY is less than MDCTDELAY, see encoder.h
@@ -1501,22 +1565,12 @@ lame_encode_buffer_sample_t(lame_global_flags * gfp,
     mfbuf[0] = gfc->mfbuf[0];
     mfbuf[1] = gfc->mfbuf[1];
 
-    if (gfp->num_channels == 2 && gfc->channels_out == 1) {
-        /* downsample to mono */
-        for (i = 0; i < nsamples; ++i) {
-            in_buffer[0][i] =
-                0.5 * ((FLOAT8) in_buffer[0][i] + in_buffer[1][i]);
-            in_buffer[1][i] = 0.0;
-        }
-    }
-
-
     while (nsamples > 0) {
         int     n_in = 0;    /* number of input samples processed with fill_buffer */
         int     n_out = 0;   /* number of samples output with fill_buffer */
         /* n_in <> n_out if we are resampling */
 
-        /* copy in new samples into mfbuf, with resampling & scaling if necessary */
+        /* copy in new samples into mfbuf, with resampling */
         fill_buffer(gfp, mfbuf, in_buffer, nsamples, &n_in, &n_out);
 
         /* update in_buffer counters */
@@ -1562,7 +1616,6 @@ lame_encode_buffer_sample_t(lame_global_flags * gfp,
     assert(nsamples == 0);
     ret = mp3size;
 
-
   retr:
     return ret;
 }
@@ -1595,7 +1648,7 @@ lame_encode_buffer(lame_global_flags * gfp,
     /* make a copy of input buffer, changing type to sample_t */
     for (i = 0; i < nsamples; i++) {
         in_buffer[0][i] = buffer_l[i];
-        in_buffer[1][i] = buffer_r[i];
+        if (gfc->channels_in>1) in_buffer[1][i] = buffer_r[i];
     }
 
     ret = lame_encode_buffer_sample_t(gfp,in_buffer[0],in_buffer[1],
@@ -1611,9 +1664,7 @@ int
 lame_encode_buffer_float(lame_global_flags * gfp,
                    const float buffer_l[],
                    const float buffer_r[],
-                   const int nsamples, 
-				   unsigned char *mp3buf, 
-				   const int mp3buf_size)
+                   const int nsamples, unsigned char *mp3buf, const int mp3buf_size)
 {
     lame_internal_flags *gfc = gfp->internal_flags;
     int     ret, i;
@@ -1636,7 +1687,7 @@ lame_encode_buffer_float(lame_global_flags * gfp,
     /* make a copy of input buffer, changing type to sample_t */
     for (i = 0; i < nsamples; i++) {
         in_buffer[0][i] = buffer_l[i];
-        in_buffer[1][i] = buffer_r[i];
+        if (gfc->channels_in>1) in_buffer[1][i] = buffer_r[i];
     }
 
     ret = lame_encode_buffer_sample_t(gfp,in_buffer[0],in_buffer[1],
@@ -1676,8 +1727,9 @@ lame_encode_buffer_int(lame_global_flags * gfp,
     /* make a copy of input buffer, changing type to sample_t */
     for (i = 0; i < nsamples; i++) {
                                 /* internal code expects +/- 32768.0 */
-      in_buffer[0][i] = buffer_l[i] * (1.0 / ( 1 << (8 * sizeof(int) - 16)));
-      in_buffer[1][i] = buffer_r[i] * (1.0 / ( 1 << (8 * sizeof(int) - 16)));
+      in_buffer[0][i] = buffer_l[i] * (1.0 / ( 1L << (8 * sizeof(int) - 16)));
+      if (gfc->channels_in>1) 
+         in_buffer[1][i] = buffer_r[i] * (1.0 / ( 1L << (8 * sizeof(int) - 16)));
     }
 
     ret = lame_encode_buffer_sample_t(gfp,in_buffer[0],in_buffer[1],
@@ -1720,7 +1772,8 @@ lame_encode_buffer_long2(lame_global_flags * gfp,
     for (i = 0; i < nsamples; i++) {
                                 /* internal code expects +/- 32768.0 */
       in_buffer[0][i] = buffer_l[i] * (1.0 / ( 1 << (8 * sizeof(long) - 16)));
-      in_buffer[1][i] = buffer_r[i] * (1.0 / ( 1 << (8 * sizeof(long) - 16)));
+      if (gfc->channels_in>1) 
+         in_buffer[1][i] = buffer_r[i] * (1.0 / ( 1 << (8 * sizeof(long) - 16)));
     }
 
     ret = lame_encode_buffer_sample_t(gfp,in_buffer[0],in_buffer[1],
@@ -1761,7 +1814,8 @@ lame_encode_buffer_long(lame_global_flags * gfp,
     /* make a copy of input buffer, changing type to sample_t */
     for (i = 0; i < nsamples; i++) {
         in_buffer[0][i] = buffer_l[i];
-        in_buffer[1][i] = buffer_r[i];
+        if (gfc->channels_in>1) 
+           in_buffer[1][i] = buffer_r[i];
     }
 
     ret = lame_encode_buffer_sample_t(gfp,in_buffer[0],in_buffer[1],
@@ -1868,13 +1922,20 @@ lame_init_bitstream(lame_global_flags * gfp)
 	id3tag_write_v2(gfp);
 
     /* initialize histogram data optionally used by frontend */
-    for ( i = 0; i < 16; i++ ) 
+    for ( i = 0; i < 16; i++ ) {
 	gfc->bitrate_stereoMode_Hist [i] [0] =
 	    gfc->bitrate_stereoMode_Hist [i] [1] =
 	    gfc->bitrate_stereoMode_Hist [i] [2] =
 	    gfc->bitrate_stereoMode_Hist [i] [3] =
 	    gfc->bitrate_stereoMode_Hist [i] [4] = 0;
-
+        gfc->bitrate_blockType_Hist [i] [0] =
+            gfc->bitrate_blockType_Hist [i] [1] =
+            gfc->bitrate_blockType_Hist [i] [2] =
+            gfc->bitrate_blockType_Hist [i] [3] =
+            gfc->bitrate_blockType_Hist [i] [4] =
+            gfc->bitrate_blockType_Hist [i] [5] = 0;
+    }
+    
     /* Write initial VBR Header to bitstream and init VBR data */
     if (gfp->bWriteVbrTag)
         InitVbrTag(gfp);
@@ -2063,6 +2124,9 @@ lame_init(void)
     lame_global_flags *gfp;
     int     ret;
 
+
+    init_log_table();
+
     gfp = calloc(1, sizeof(lame_global_flags));
     if (gfp == NULL)
         return NULL;
@@ -2170,6 +2234,7 @@ lame_init_old(lame_global_flags * gfp)
     gfp->asm_optimizations.amd3dnow = 1;
     gfp->asm_optimizations.sse = 1;
 
+    gfp->preset = 0;
     return 0;
 }
 
@@ -2255,10 +2320,7 @@ lame_stereo_mode_hist(const lame_global_flags * const gfp, int stmode_count[4])
         return;
 
     for (i = 0; i < 4; i++) {
-        int     j, sum = 0;
-        for (j = 0; j < 14; j++)
-            sum += gfc->bitrate_stereoMode_Hist[j + 1][i];
-        stmode_count[i] = sum;
+        stmode_count[i] = gfc->bitrate_stereoMode_Hist[15][i];
     }
 }
 
@@ -2285,4 +2347,47 @@ lame_bitrate_stereo_mode_hist(const lame_global_flags * const gfp,
             bitrate_stmode_count[j][i] = gfc->bitrate_stereoMode_Hist[j + 1][i];
 }
 
+
+
+void
+lame_block_type_hist(const lame_global_flags * const gfp, int btype_count[6])
+{
+    const lame_internal_flags *gfc;
+    int     i;
+
+    if (NULL == btype_count)
+        return;
+    if (NULL == gfp)
+        return;
+    gfc = gfp->internal_flags;
+    if (NULL == gfc)
+        return;
+
+    for (i = 0; i < 6; ++i) {
+        btype_count[i] = gfc->bitrate_blockType_Hist[15][i];
+    }
+}
+
+
+
+void 
+lame_bitrate_block_type_hist(const lame_global_flags * const gfp, 
+                             int bitrate_btype_count[14][6])
+{
+    const lame_internal_flags * gfc;
+    int     i, j;
+    
+    if (NULL == bitrate_btype_count)
+        return;
+    if (NULL == gfp)
+        return;
+    gfc = gfp->internal_flags;
+    if (NULL == gfc)
+        return;
+        
+    for (j = 0; j < 14; ++j)
+    for (i = 0; i <  6; ++i)
+        bitrate_btype_count[j][i] = gfc->bitrate_blockType_Hist[j+1][i];
+}
+
 /* end of lame.c */
