@@ -20,7 +20,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* $Id: set_get.c,v 1.1.1.1 2002/06/21 12:35:24 myrina Exp $ */
+/* $Id: set_get.c,v 1.1 2003/07/29 04:17:52 heroine Exp $ */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -35,6 +35,12 @@
 #endif
 
 
+
+extern int apply_preset(lame_global_flags*  gfp, int preset);
+
+
+
+
 /*
  * input stream description
  */
@@ -472,6 +478,10 @@ lame_set_brate( lame_global_flags*  gfp,
 {
     gfp->brate = brate;
 
+    if (brate >= 320) {
+        gfp->disable_reservoir = 1;
+    }
+
     return 0;
 }
 
@@ -1264,7 +1274,7 @@ lame_set_interChRatio( lame_global_flags*  gfp,
     return 0;
 }
 
-int
+float
 lame_get_interChRatio( const lame_global_flags*  gfp )
 {
     assert( 0 <= gfp->interChRatio && gfp->interChRatio <= 1.0);
@@ -1273,6 +1283,28 @@ lame_get_interChRatio( const lame_global_flags*  gfp )
 }
 
 
+/* Use pseudo substep shaping method */
+int
+lame_set_substep( lame_global_flags*  gfp,
+		  int                 method )
+{
+    lame_internal_flags *gfc = gfp->internal_flags;
+    /* default = 0.0 (no inter-cahnnel maskin) */
+    if (! (0 <= method && method <= 2))
+        return -1;
+
+    gfc->substep_shaping = method;
+    return 0;
+}
+
+int
+lame_get_substep(const lame_global_flags*  gfp )
+{
+    lame_internal_flags *gfc = gfp->internal_flags;
+    assert(0 <= gfc->substep_shaping && gfc->substep_shaping <= 2);
+    return gfc->substep_shaping;
+}
+
 /* Disable short blocks. */
 int
 lame_set_no_short_blocks( lame_global_flags*  gfp,
@@ -1593,6 +1625,7 @@ lame_set_preset_notune( lame_global_flags*  gfp, int preset_notune )
 int
 lame_set_preset( lame_global_flags*  gfp, int preset )
 {
+    gfp->preset = preset;
     return apply_preset(gfp, preset);
 }
 
