@@ -19,7 +19,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* $Id: takehiro.c,v 1.1.1.1 2002/06/21 12:35:24 myrina Exp $ */
+/* $Id: takehiro.c,v 1.1 2003/07/29 04:17:52 heroine Exp $ */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -332,34 +332,35 @@ int count_bits(
     else
 	quantize_xrpow_ISO(xr, ix, IPOW20(gi->global_gain));
 
-    if (gfc->noise_shaping_amp==3) {
-      int sfb;
+    if (gfc->substep_shaping) {
+      int sfb, j = 0;
       // 0.634521682242439 = 0.5946*2**(.5*0.1875)
-      FLOAT8 roundfac = 0.634521682242439 / IPOW20(gi->global_gain+gi->scalefac_scale);
-      i = 0;
+      const FLOAT8 roundfac =
+	  0.634521682242439 / IPOW20(gi->global_gain+gi->scalefac_scale);
       for (sfb = 0; sfb < gi->sfb_lmax; sfb++) {
-	int end;
-	if (!gfc->pseudohalf.l[sfb])
-	  continue;
+	  int width = gfc->scalefac_band.l[sfb+1] - gfc->scalefac_band.l[sfb];
+	  int l;
+	  j += width;
+	  if (!gfc->pseudohalf.l[sfb])
+	      continue;
 
-	end   = gfc->scalefac_band.l[sfb+1];
-	for (; i < end; i++)
-	  if (xr[i] < roundfac)
-	    ix[i] = 0;
+	  for (l = -width; l < 0; l++)
+	      if (xr[j+l] < roundfac)
+		  ix[j+l] = 0;
       }
 
       for (sfb = gi->sfb_smin; sfb < SBPSY_s; sfb++) {
-	int start, end, win;
-	start = gfc->scalefac_band.s[sfb];
-	end   = gfc->scalefac_band.s[sfb+1];
-	for (win = 0; win < 3; win++) {
-	  int j;
-	  if (!gfc->pseudohalf.s[sfb][win])
-	    continue;
-	  for (j = start; j < end; j++, i++)
-	    if (xr[i] < roundfac)
-	      ix[i] = 0;
-	}
+	  int b;
+	  int width = gfc->scalefac_band.s[sfb+1] - gfc->scalefac_band.s[sfb];
+	  for (b = 0; b < 3; b++) {
+	      int l;
+	      j += width;
+	      if (!gfc->pseudohalf.s[sfb][b])
+		  continue;
+	      for (l = -width; l < 0; l++)
+		  if (xr[j+l] < roundfac)
+		      ix[j+l] = 0;
+	  }
       }
     }
 
@@ -453,7 +454,7 @@ int count_bits(
 inline static void
 recalc_divide_init(
     const lame_internal_flags * const gfc,
-          gr_info         cod_info,
+          gr_info         *cod_info,
           int     * const ix,
           int             r01_bits[],
           int             r01_div [],
@@ -462,7 +463,7 @@ recalc_divide_init(
 {
     int r0, r1, bigv, r0t, r1t, bits;
 
-    bigv = cod_info.big_values;
+    bigv = cod_info->big_values;
 
     for (r0 = 0; r0 <= 7 + 15; r0++) {
 	r01_bits[r0] = LARGE_BITS;
@@ -472,7 +473,7 @@ recalc_divide_init(
 	int a1 = gfc->scalefac_band.l[r0 + 1], r0bits;
 	if (a1 >= bigv)
 	    break;
-	r0bits = cod_info.part2_length;
+	r0bits = cod_info->part2_length;
 	r0t = gfc->choose_table(ix, ix + a1, &r0bits);
 
 	for (r1 = 0; r1 < 8; r1++) {
@@ -495,7 +496,7 @@ recalc_divide_init(
 inline static void
 recalc_divide_sub(
     const lame_internal_flags * const gfc,
-    const gr_info         cod_info2,
+    const gr_info         *cod_info2,
           gr_info * const gi,
     const int     * const ix,
     const int             r01_bits[],
@@ -505,14 +506,14 @@ recalc_divide_sub(
 {
     int bits, r2, a2, bigv, r2t;
 
-    bigv = cod_info2.big_values;
+    bigv = cod_info2->big_values;
 
     for (r2 = 2; r2 < SBMAX_l + 1; r2++) {
 	a2 = gfc->scalefac_band.l[r2];
 	if (a2 >= bigv) 
 	    break;
 
-	bits = r01_bits[r2 - 2] + cod_info2.count1bits;
+	bits = r01_bits[r2 - 2] + cod_info2->count1bits;
 	if (gi->part2_3_length <= bits)
 	    break;
 
@@ -520,7 +521,7 @@ recalc_divide_sub(
 	if (gi->part2_3_length <= bits)
 	    continue;
 
-	memcpy(gi, &cod_info2, sizeof(gr_info));
+	memcpy(gi, cod_info2, sizeof(gr_info));
 	gi->part2_3_length = bits;
 	gi->region0_count = r01_div[r2 - 2];
 	gi->region1_count = r2 - 2 - r01_div[r2 - 2];
@@ -535,11 +536,11 @@ recalc_divide_sub(
 
 void best_huffman_divide(
     const lame_internal_flags * const gfc,
-          gr_info * const gi,
-          int     * const ix )
+    gr_info * const gi)
 {
     int i, a1, a2;
     gr_info cod_info2;
+    int * const ix = gi->l3_enc;
 
     int r01_bits[7 + 15 + 1];
     int r01_div[7 + 15 + 1];
@@ -554,8 +555,8 @@ void best_huffman_divide(
 
     memcpy(&cod_info2, gi, sizeof(gr_info));
     if (gi->block_type == NORM_TYPE) {
-	recalc_divide_init(gfc, cod_info2, ix, r01_bits,r01_div,r0_tbl,r1_tbl);
-	recalc_divide_sub(gfc, cod_info2, gi, ix, r01_bits,r01_div,r0_tbl,r1_tbl);
+	recalc_divide_init(gfc, gi, ix, r01_bits,r01_div,r0_tbl,r1_tbl);
+	recalc_divide_sub(gfc, &cod_info2, gi, ix, r01_bits,r01_div,r0_tbl,r1_tbl);
     }
 
     i = cod_info2.big_values;
@@ -587,12 +588,12 @@ void best_huffman_divide(
     }
 
     cod_info2.count1bits = a1;
-    cod_info2.part2_3_length = a1 + cod_info2.part2_length;
 
     if (cod_info2.block_type == NORM_TYPE)
-	recalc_divide_sub(gfc, cod_info2, gi, ix, r01_bits,r01_div,r0_tbl,r1_tbl);
+	recalc_divide_sub(gfc, &cod_info2, gi, ix, r01_bits,r01_div,r0_tbl,r1_tbl);
     else {
 	/* Count the number of bits necessary to code the bigvalues region. */
+	cod_info2.part2_3_length = a1 + cod_info2.part2_length;
 	a1 = gfc->scalefac_band.l[7 + 1];
 	if (a1 > i) {
 	    a1 = i;
@@ -613,12 +614,12 @@ static const int slen2_n[16] = { 1, 2, 4, 8, 1, 2, 4, 8, 2, 4, 8, 2, 4, 8, 4, 8
 
 void
 scfsi_calc(int ch,
-	   III_side_info_t *l3_side,
-	   III_scalefac_t scalefac[2][2])
+	   III_side_info_t *l3_side)
 {
     int i, s1, s2, c1, c2;
     int sfb;
-    gr_info *gi = &l3_side->gr[1].ch[ch].tt;
+    gr_info *gi = &l3_side->tt[1][ch];
+    gr_info *g0 = &l3_side->tt[0][ch];
 
     static const int scfsi_band[5] = { 0, 6, 11, 16, 21 };
 #if 0
@@ -631,12 +632,12 @@ scfsi_calc(int ch,
 
     for (i = 0; i < (sizeof(scfsi_band) / sizeof(int)) - 1; i++) {
 	for (sfb = scfsi_band[i]; sfb < scfsi_band[i + 1]; sfb++) {
-	    if (scalefac[0][ch].l[sfb] != scalefac[1][ch].l[sfb])
+	    if (g0->scalefac.l[sfb] != gi->scalefac.l[sfb])
 		break;
 	}
 	if (sfb == scfsi_band[i + 1]) {
 	    for (sfb = scfsi_band[i]; sfb < scfsi_band[i + 1]; sfb++) {
-		scalefac[1][ch].l[sfb] = -1;
+		gi->scalefac.l[sfb] = -1;
 	    }
 	    l3_side->scfsi[ch][i] = 1;
 	}
@@ -644,20 +645,20 @@ scfsi_calc(int ch,
 
     s1 = c1 = 0;
     for (sfb = 0; sfb < 11; sfb++) {
-	if (scalefac[1][ch].l[sfb] < 0)
+	if (gi->scalefac.l[sfb] < 0)
 	    continue;
 	c1++;
-	if (s1 < scalefac[1][ch].l[sfb])
-	    s1 = scalefac[1][ch].l[sfb];
+	if (s1 < gi->scalefac.l[sfb])
+	    s1 = gi->scalefac.l[sfb];
     }
 
     s2 = c2 = 0;
     for (; sfb < SBPSY_l; sfb++) {
-	if (scalefac[1][ch].l[sfb] < 0)
+	if (gi->scalefac.l[sfb] < 0)
 	    continue;
 	c2++;
-	if (s2 < scalefac[1][ch].l[sfb])
-	    s2 = scalefac[1][ch].l[sfb];
+	if (s2 < gi->scalefac.l[sfb])
+	    s2 = gi->scalefac.l[sfb];
     }
 
     for (i = 0; i < 16; i++) {
@@ -680,70 +681,67 @@ void best_scalefac_store(
     const lame_internal_flags *gfc,
     const int             gr,
     const int             ch,
-          int             l3_enc[2][2][576],
-          III_side_info_t * const l3_side,
-          III_scalefac_t          scalefac[2][2] )
+          III_side_info_t * const l3_side)
 {
 
     /* use scalefac_scale if we can */
-    gr_info *gi = &l3_side->gr[gr].ch[ch].tt;
-    int sfb,i,j,j2,l,start,end;
+    gr_info *gi = &l3_side->tt[gr][ch];
+    int sfb,i,j,l;
 
     /* remove scalefacs from bands with ix=0.  This idea comes
      * from the AAC ISO docs.  added mt 3/00 */
     /* check if l3_enc=0 */
+    j = 0;
     for ( sfb = 0; sfb < gi->sfb_lmax; sfb++ ) {
-      if (scalefac[gr][ch].l[sfb]>0) { 
-	start = gfc->scalefac_band.l[ sfb ];
-	end   = gfc->scalefac_band.l[ sfb+1 ];
-	for ( l = start; l < end; l++ ) if (l3_enc[gr][ch][l]!=0) break;
-	if (l==end) scalefac[gr][ch].l[sfb]=0;
-      }
+	int width = gfc->scalefac_band.l[sfb+1] - gfc->scalefac_band.l[sfb];
+	j += width;
+	if (gi->scalefac.l[sfb] == 0)
+	    continue;
+
+	for (l = -width; l < 0; l++) if (gi->l3_enc[l+j]!=0) break;
+	if (l==0) gi->scalefac.l[sfb]=0;
     }
-    for ( j=0, sfb = gi->sfb_smin; sfb < SBPSY_s; sfb++ ) {
-	start = gfc->scalefac_band.s[ sfb ];
-	end   = gfc->scalefac_band.s[ sfb+1 ];
+    for (sfb = gi->sfb_smin; sfb < SBPSY_s; sfb++ ) {
+	int width = gfc->scalefac_band.s[sfb+1] - gfc->scalefac_band.s[sfb];
 	for ( i = 0; i < 3; i++ ) {
-	  if (scalefac[gr][ch].s[sfb][i]>0) {
-	    j2 = j;
-	    for ( l = start; l < end; l++ ) 
-	      if (l3_enc[gr][ch][j2++ /*3*l+i*/]!=0) break;
-	    if (l==end) scalefac[gr][ch].s[sfb][i]=0;
-	  }
-	  j += end-start;
+	    j += width;
+	    if (gi->scalefac.s[sfb][i] == 0)
+		continue;
+
+	    for (l = -width; l < 0; l++) if (gi->l3_enc[l+j]!=0) break;
+	    if (l==0) gi->scalefac.s[sfb][i]=0;
 	}
     }
 
-
     gi->part2_3_length -= gi->part2_length;
     if (!gi->scalefac_scale && !gi->preflag) {
 	int b, s = 0;
 	for (sfb = 0; sfb < gi->sfb_lmax; sfb++) {
-	    s |= scalefac[gr][ch].l[sfb];
+	    s |= gi->scalefac.l[sfb];
 	}
 
 	for (sfb = gi->sfb_smin; sfb < SBPSY_s; sfb++) {
 	    for (b = 0; b < 3; b++) {
-		s |= scalefac[gr][ch].s[sfb][b];
+		s |= gi->scalefac.s[sfb][b];
 	    }
 	}
 
 	if (!(s & 1) && s != 0) {
 	    for (sfb = 0; sfb < gi->sfb_lmax; sfb++) {
-		scalefac[gr][ch].l[sfb] /= 2;
+		gi->scalefac.l[sfb] /= 2;
 	    }
 	    for (sfb = gi->sfb_smin; sfb < SBPSY_s; sfb++) {
 		for (b = 0; b < 3; b++) {
-		    scalefac[gr][ch].s[sfb][b] /= 2;
+		    gi->scalefac.s[sfb][b] /= 2;
 		}
 	    }
 
 	    gi->scalefac_scale = 1;
 	    gi->part2_length = 99999999;
 	    if (gfc->mode_gr == 2) {
-	        scale_bitcount(&scalefac[gr][ch], gi);
+	        scale_bitcount(&gi->scalefac, gi);
 	    } else {
-		scale_bitcount_lsf(gfc,&scalefac[gr][ch], gi);
+		scale_bitcount_lsf(gfc,&gi->scalefac, gi);
 	    }
 	}
     }
@@ -753,9 +751,9 @@ void best_scalefac_store(
       l3_side->scfsi[ch][i] = 0;
 
     if (gfc->mode_gr==2 && gr == 1
-	&& l3_side->gr[0].ch[ch].tt.block_type != SHORT_TYPE
-	&& l3_side->gr[1].ch[ch].tt.block_type != SHORT_TYPE) {
-      	scfsi_calc(ch, l3_side, scalefac);
+	&& l3_side->tt[0][ch].block_type != SHORT_TYPE
+	&& l3_side->tt[1][ch].block_type != SHORT_TYPE) {
+      	scfsi_calc(ch, l3_side);
     }
     gi->part2_3_length += gi->part2_length;
 }
