@@ -33,7 +33,14 @@
 #include "lame_global_flags.h"
 #include "lame-analysis.h"
 #include "id3tag.h"
-#include <stdint.h>
+
+#if HAVE_INTTYPES_H
+# include <inttypes.h>
+#else
+# if HAVE_STDINT_H
+#  include <stdint.h>
+# endif
+#endif
 
 #ifdef __cplusplus
 extern "C" {
@@ -98,8 +105,18 @@ extern "C" {
 #define         Min(A, B)       ((A) < (B) ? (A) : (B))
 #define         Max(A, B)       ((A) > (B) ? (A) : (B))
 
+/* log/log10 approximations */
+#ifdef TAKEHIRO_IEEE754_HACK
+#define USE_FAST_LOG
+#endif
 
-
+#ifdef USE_FAST_LOG
+#define         FAST_LOG10(x)       (fast_log2(x)*(LOG2/LOG10))
+#define         FAST_LOG(x)         (fast_log2(x)*LOG2)
+#else
+#define         FAST_LOG10(x)       log10(x)
+#define         FAST_LOG(x)         log(x)
+#endif
 
 
 /***********************************************************************
@@ -195,14 +212,15 @@ typedef struct
  */
 typedef struct
 {
-    int     use_adjust;     // method for the auto adjustment 
-    FLOAT8  adjust;         // lowering based on peak volume, 1 = no lowering
-    FLOAT8  adjust_limit;   // limit for dynamic ATH adjust
-    FLOAT8  decay;          // determined to lower x dB each second
-    FLOAT8  floor;          // lowest ATH value
-    FLOAT8  l[SBMAX_l];     // ATH for sfbs in long blocks
-    FLOAT8  s[SBMAX_s];     // ATH for sfbs in short blocks
-    FLOAT8  cb[CBANDS];     // ATH for convolution bands
+    int     use_adjust;     /* method for the auto adjustment */
+    FLOAT8  adjust;         /* lowering based on peak volume, 1 = no lowering */
+    FLOAT8  adjust_limit;   /* limit for dynamic ATH adjust */
+    FLOAT8  decay;          /* determined to lower x dB each second */
+    FLOAT8  floor;          /* lowest ATH value */
+    FLOAT8  l[SBMAX_l];     /* ATH for sfbs in long blocks */
+    FLOAT8  s[SBMAX_s];     /* ATH for sfbs in short blocks */
+    FLOAT8  cb[CBANDS];     /* ATH for convolution bands */
+    FLOAT   eql_w[BLKSIZE/2];	/* equal loudness weights (based on ATH) */
 } ATH_t;
 
 /**
@@ -210,11 +228,10 @@ typedef struct
  */
 typedef struct
 {
-    int     quality;
-    FLOAT8  mask_adjust;    // the dbQ stuff
-    int     smooth;         // 0=no, 1=peaks, 2=+-4
-    int     bitpressure;    // strategy for bitpressure situations
     FLOAT8  scratch[192];
+    FLOAT8  mask_adjust;    // the dbQ stuff
+    int     quality;
+    int     smooth;         // 0=no, 1=peaks, 2=+-4
 } VBR_t;
 
 /**
@@ -360,11 +377,10 @@ struct lame_internal_flags {
   int noise_shaping_amp;    /*  0 = ISO model: amplify all distorted bands
                                 1 = amplify within 50% of max (on db scale)
                                 2 = amplify only most distorted band
-                                3 = amplify only most distorted band and
-				    use pseudo half step
 			     */
   int substep_shaping;      /* 0 = no substep
 			       1 = use substep shaping at last step(VBR only)
+			           (not implemented yet)
 			       2 = use substep inside loop
 			    */
 
@@ -519,6 +535,7 @@ struct lame_internal_flags {
   
   /* simple statistics */
   int   bitrate_stereoMode_Hist [16] [4+1];
+  int	bitrate_blockType_Hist  [16] [4+1+1];/*norm/start/short/stop/mixed(short)/sum*/
 
   /* ratios  */
   FLOAT8 pe[4];
@@ -575,12 +592,17 @@ extern int            BitrateIndex(int, int,int);
 extern int            FindNearestBitrate(int,int,int);
 extern int            map2MP3Frequency(int freq);
 extern int            SmpFrqIndex(int, int* const);
+extern int            nearestBitrateFullIndex(const int brate);
 extern FLOAT8         ATHformula(FLOAT8 f,lame_global_flags *gfp);
 extern FLOAT8         freq2bark(FLOAT8 freq);
 extern FLOAT8         freq2cbw(FLOAT8 freq);
-extern void freorder(int scalefac_band[],FLOAT8 ix_orig[576]);
 void disable_FPE(void);
 
+/* log/log10 approximations */
+extern void init_log_table(void);
+extern ieee754_float32_t fast_log2(ieee754_float32_t x);
+
+
 extern void 
 getframebits( const lame_global_flags *gfp, int *bitsPerFrame, int *mean_bits);
 
