@@ -5,14 +5,14 @@
  * GOVERNED BY A BSD-STYLE SOURCE LICENSE INCLUDED WITH THIS SOURCE *
  * IN 'COPYING'. PLEASE READ THESE TERMS BEFORE DISTRIBUTING.       *
  *                                                                  *
- * THE OggVorbis SOURCE CODE IS (C) COPYRIGHT 1994-2001             *
- * by the XIPHOPHORUS Company http://www.xiph.org/                  *
-
+ * THE OggVorbis SOURCE CODE IS (C) COPYRIGHT 1994-2002             *
+ * by the Xiph.Org Foundation http://www.xiph.org/                  *
+ *                                                                  *
  ********************************************************************
 
  function: code raw [Vorbis] packets into framed OggSquish stream and
            decode Ogg streams back into raw packets
- last mod: $Id: framing.c,v 1.1.1.1 2002/06/21 12:35:23 myrina Exp $
+ last mod: $Id: framing.c,v 1.1 2002/07/22 12:33:53 myrina Exp $
 
  note: The CRC code is directly derived from public domain code by
  Ross Williams (ross@guest.adelaide.edu.au).  See docs/framing.html
@@ -91,15 +91,14 @@ more page packet), the return will be:
 int ogg_page_packets(ogg_page *og){
   int i,n=og->header[26],count=0;
   for(i=0;i<n;i++)
-    if(og->header[27+i]==0)count++;
+    if(og->header[27+i]<255)count++;
   return(count);
 }
 
 
-/* helper to initialize lookup for direct-table CRC */
-
-static ogg_uint32_t crc_lookup[256];
-static int crc_ready=0;
+#if 0
+/* helper to initialize lookup for direct-table CRC (illustrative; we
+   use the static init below) */
 
 static ogg_uint32_t _ogg_crc_entry(unsigned long index){
   int           i;
@@ -116,34 +115,85 @@ static ogg_uint32_t _ogg_crc_entry(unsigned long index){
        r<<=1;
  return (r & 0xffffffffUL);
 }
+#endif
 
-/* mind this in threaded code; sync_init and stream_init call it.
-   It's thread safe only after the first time it returns */
-
-static void _ogg_crc_init(void){
-  if(!crc_ready){
-    /* initialize the crc_lookup table */
-    int i;
-    for (i=0;i<256;i++)
-      crc_lookup[i]=_ogg_crc_entry((unsigned long)i);
-    crc_ready=0;
-  }
-}
+static ogg_uint32_t crc_lookup[256]={
+  0x00000000,0x04c11db7,0x09823b6e,0x0d4326d9,
+  0x130476dc,0x17c56b6b,0x1a864db2,0x1e475005,
+  0x2608edb8,0x22c9f00f,0x2f8ad6d6,0x2b4bcb61,
+  0x350c9b64,0x31cd86d3,0x3c8ea00a,0x384fbdbd,
+  0x4c11db70,0x48d0c6c7,0x4593e01e,0x4152fda9,
+  0x5f15adac,0x5bd4b01b,0x569796c2,0x52568b75,
+  0x6a1936c8,0x6ed82b7f,0x639b0da6,0x675a1011,
+  0x791d4014,0x7ddc5da3,0x709f7b7a,0x745e66cd,
+  0x9823b6e0,0x9ce2ab57,0x91a18d8e,0x95609039,
+  0x8b27c03c,0x8fe6dd8b,0x82a5fb52,0x8664e6e5,
+  0xbe2b5b58,0xbaea46ef,0xb7a96036,0xb3687d81,
+  0xad2f2d84,0xa9ee3033,0xa4ad16ea,0xa06c0b5d,
+  0xd4326d90,0xd0f37027,0xddb056fe,0xd9714b49,
+  0xc7361b4c,0xc3f706fb,0xceb42022,0xca753d95,
+  0xf23a8028,0xf6fb9d9f,0xfbb8bb46,0xff79a6f1,
+  0xe13ef6f4,0xe5ffeb43,0xe8bccd9a,0xec7dd02d,
+  0x34867077,0x30476dc0,0x3d044b19,0x39c556ae,
+  0x278206ab,0x23431b1c,0x2e003dc5,0x2ac12072,
+  0x128e9dcf,0x164f8078,0x1b0ca6a1,0x1fcdbb16,
+  0x018aeb13,0x054bf6a4,0x0808d07d,0x0cc9cdca,
+  0x7897ab07,0x7c56b6b0,0x71159069,0x75d48dde,
+  0x6b93dddb,0x6f52c06c,0x6211e6b5,0x66d0fb02,
+  0x5e9f46bf,0x5a5e5b08,0x571d7dd1,0x53dc6066,
+  0x4d9b3063,0x495a2dd4,0x44190b0d,0x40d816ba,
+  0xaca5c697,0xa864db20,0xa527fdf9,0xa1e6e04e,
+  0xbfa1b04b,0xbb60adfc,0xb6238b25,0xb2e29692,
+  0x8aad2b2f,0x8e6c3698,0x832f1041,0x87ee0df6,
+  0x99a95df3,0x9d684044,0x902b669d,0x94ea7b2a,
+  0xe0b41de7,0xe4750050,0xe9362689,0xedf73b3e,
+  0xf3b06b3b,0xf771768c,0xfa325055,0xfef34de2,
+  0xc6bcf05f,0xc27dede8,0xcf3ecb31,0xcbffd686,
+  0xd5b88683,0xd1799b34,0xdc3abded,0xd8fba05a,
+  0x690ce0ee,0x6dcdfd59,0x608edb80,0x644fc637,
+  0x7a089632,0x7ec98b85,0x738aad5c,0x774bb0eb,
+  0x4f040d56,0x4bc510e1,0x46863638,0x42472b8f,
+  0x5c007b8a,0x58c1663d,0x558240e4,0x51435d53,
+  0x251d3b9e,0x21dc2629,0x2c9f00f0,0x285e1d47,
+  0x36194d42,0x32d850f5,0x3f9b762c,0x3b5a6b9b,
+  0x0315d626,0x07d4cb91,0x0a97ed48,0x0e56f0ff,
+  0x1011a0fa,0x14d0bd4d,0x19939b94,0x1d528623,
+  0xf12f560e,0xf5ee4bb9,0xf8ad6d60,0xfc6c70d7,
+  0xe22b20d2,0xe6ea3d65,0xeba91bbc,0xef68060b,
+  0xd727bbb6,0xd3e6a601,0xdea580d8,0xda649d6f,
+  0xc423cd6a,0xc0e2d0dd,0xcda1f604,0xc960ebb3,
+  0xbd3e8d7e,0xb9ff90c9,0xb4bcb610,0xb07daba7,
+  0xae3afba2,0xaafbe615,0xa7b8c0cc,0xa379dd7b,
+  0x9b3660c6,0x9ff77d71,0x92b45ba8,0x9675461f,
+  0x8832161a,0x8cf30bad,0x81b02d74,0x857130c3,
+  0x5d8a9099,0x594b8d2e,0x5408abf7,0x50c9b640,
+  0x4e8ee645,0x4a4ffbf2,0x470cdd2b,0x43cdc09c,
+  0x7b827d21,0x7f436096,0x7200464f,0x76c15bf8,
+  0x68860bfd,0x6c47164a,0x61043093,0x65c52d24,
+  0x119b4be9,0x155a565e,0x18197087,0x1cd86d30,
+  0x029f3d35,0x065e2082,0x0b1d065b,0x0fdc1bec,
+  0x3793a651,0x3352bbe6,0x3e119d3f,0x3ad08088,
+  0x2497d08d,0x2056cd3a,0x2d15ebe3,0x29d4f654,
+  0xc5a92679,0xc1683bce,0xcc2b1d17,0xc8ea00a0,
+  0xd6ad50a5,0xd26c4d12,0xdf2f6bcb,0xdbee767c,
+  0xe3a1cbc1,0xe760d676,0xea23f0af,0xeee2ed18,
+  0xf0a5bd1d,0xf464a0aa,0xf9278673,0xfde69bc4,
+  0x89b8fd09,0x8d79e0be,0x803ac667,0x84fbdbd0,
+  0x9abc8bd5,0x9e7d9662,0x933eb0bb,0x97ffad0c,
+  0xafb010b1,0xab710d06,0xa6322bdf,0xa2f33668,
+  0xbcb4666d,0xb8757bda,0xb5365d03,0xb1f740b4};
 
 /* init the encode/decode logical stream state */
 
 int ogg_stream_init(ogg_stream_state *os,int serialno){
   if(os){
-    memset(os,0,sizeof(ogg_stream_state));
+    memset(os,0,sizeof(*os));
     os->body_storage=16*1024;
-    os->body_data=_ogg_malloc(os->body_storage*sizeof(char));
+    os->body_data=_ogg_malloc(os->body_storage*sizeof(*os->body_data));
 
     os->lacing_storage=1024;
-    os->lacing_vals=_ogg_malloc(os->lacing_storage*sizeof(int));
-    os->granule_vals=_ogg_malloc(os->lacing_storage*sizeof(ogg_int64_t));
-
-    /* initialize the crc_lookup table if not done */
-    _ogg_crc_init();
+    os->lacing_vals=_ogg_malloc(os->lacing_storage*sizeof(*os->lacing_vals));
+    os->granule_vals=_ogg_malloc(os->lacing_storage*sizeof(*os->granule_vals));
 
     os->serialno=serialno;
 
@@ -159,7 +209,7 @@ int ogg_stream_clear(ogg_stream_state *os){
     if(os->lacing_vals)_ogg_free(os->lacing_vals);
     if(os->granule_vals)_ogg_free(os->granule_vals);
 
-    memset(os,0,sizeof(ogg_stream_state));    
+    memset(os,0,sizeof(*os));    
   }
   return(0);
 } 
@@ -178,15 +228,15 @@ int ogg_stream_destroy(ogg_stream_state *os){
 static void _os_body_expand(ogg_stream_state *os,int needed){
   if(os->body_storage<=os->body_fill+needed){
     os->body_storage+=(needed+1024);
-    os->body_data=_ogg_realloc(os->body_data,os->body_storage);
+    os->body_data=_ogg_realloc(os->body_data,os->body_storage*sizeof(*os->body_data));
   }
 }
 
 static void _os_lacing_expand(ogg_stream_state *os,int needed){
   if(os->lacing_storage<=os->lacing_fill+needed){
     os->lacing_storage+=(needed+32);
-    os->lacing_vals=_ogg_realloc(os->lacing_vals,os->lacing_storage*sizeof(int));
-    os->granule_vals=_ogg_realloc(os->granule_vals,os->lacing_storage*sizeof(ogg_int64_t));
+    os->lacing_vals=_ogg_realloc(os->lacing_vals,os->lacing_storage*sizeof(*os->lacing_vals));
+    os->granule_vals=_ogg_realloc(os->granule_vals,os->lacing_storage*sizeof(*os->granule_vals));
   }
 }
 
@@ -194,19 +244,27 @@ static void _os_lacing_expand(ogg_stream_state *os,int needed){
 /* Direct table CRC; note that this will be faster in the future if we
    perform the checksum silmultaneously with other copies */
 
-static void _os_checksum(ogg_page *og){
-  ogg_uint32_t crc_reg=0;
-  int i;
+void ogg_page_checksum_set(ogg_page *og){
+  if(og){
+    ogg_uint32_t crc_reg=0;
+    int i;
 
-  for(i=0;i<og->header_len;i++)
-    crc_reg=(crc_reg<<8)^crc_lookup[((crc_reg >> 24)&0xff)^og->header[i]];
-  for(i=0;i<og->body_len;i++)
-    crc_reg=(crc_reg<<8)^crc_lookup[((crc_reg >> 24)&0xff)^og->body[i]];
-  
-  og->header[22]=crc_reg&0xff;
-  og->header[23]=(crc_reg>>8)&0xff;
-  og->header[24]=(crc_reg>>16)&0xff;
-  og->header[25]=(crc_reg>>24)&0xff;
+    /* safety; needed for API behavior, but not framing code */
+    og->header[22]=0;
+    og->header[23]=0;
+    og->header[24]=0;
+    og->header[25]=0;
+    
+    for(i=0;i<og->header_len;i++)
+      crc_reg=(crc_reg<<8)^crc_lookup[((crc_reg >> 24)&0xff)^og->header[i]];
+    for(i=0;i<og->body_len;i++)
+      crc_reg=(crc_reg<<8)^crc_lookup[((crc_reg >> 24)&0xff)^og->body[i]];
+    
+    og->header[22]=crc_reg&0xff;
+    og->header[23]=(crc_reg>>8)&0xff;
+    og->header[24]=(crc_reg>>16)&0xff;
+    og->header[25]=(crc_reg>>24)&0xff;
+  }
 }
 
 /* submit data to the internal buffer of the framing engine */
@@ -221,7 +279,7 @@ int ogg_stream_packetin(ogg_stream_state *os,ogg_packet *op){
     os->body_fill-=os->body_returned;
     if(os->body_fill)
       memmove(os->body_data,os->body_data+os->body_returned,
-	      os->body_fill*sizeof(char));
+	      os->body_fill);
     os->body_returned=0;
   }
  
@@ -368,13 +426,13 @@ int ogg_stream_flush(ogg_stream_state *os,ogg_page *og){
   /* advance the lacing data and set the body_returned pointer */
   
   os->lacing_fill-=vals;
-  memmove(os->lacing_vals,os->lacing_vals+vals,os->lacing_fill*sizeof(int));
-  memmove(os->granule_vals,os->granule_vals+vals,os->lacing_fill*sizeof(ogg_int64_t));
+  memmove(os->lacing_vals,os->lacing_vals+vals,os->lacing_fill*sizeof(*os->lacing_vals));
+  memmove(os->granule_vals,os->granule_vals+vals,os->lacing_fill*sizeof(*os->granule_vals));
   os->body_returned+=bytes;
   
   /* calculate the checksum */
   
-  _os_checksum(og);
+  ogg_page_checksum_set(og);
 
   /* done */
   return(1);
@@ -422,8 +480,7 @@ int ogg_stream_eos(ogg_stream_state *os){
 /* initialize the struct to a known state */
 int ogg_sync_init(ogg_sync_state *oy){
   if(oy){
-    memset(oy,0,sizeof(ogg_sync_state));
-    _ogg_crc_init();
+    memset(oy,0,sizeof(*oy));
   }
   return(0);
 }
@@ -451,8 +508,7 @@ char *ogg_sync_buffer(ogg_sync_state *oy, long size){
   if(oy->returned){
     oy->fill-=oy->returned;
     if(oy->fill>0)
-      memmove(oy->data,oy->data+oy->returned,
-	      (oy->fill)*sizeof(char));
+      memmove(oy->data,oy->data+oy->returned,oy->fill);
     oy->returned=0;
   }
 
@@ -525,7 +581,7 @@ long ogg_sync_pageseek(ogg_sync_state *oy,ogg_page *og){
     log.header_len=oy->headerbytes;
     log.body=page+oy->headerbytes;
     log.body_len=oy->bodybytes;
-    _os_checksum(&log);
+    ogg_page_checksum_set(&log);
     
     /* Compare */
     if(memcmp(chksum,page+22,4)){
@@ -646,9 +702,9 @@ int ogg_stream_pagein(ogg_stream_state *os, ogg_page *og){
       /* segment table */
       if(os->lacing_fill-lr){
 	memmove(os->lacing_vals,os->lacing_vals+lr,
-		(os->lacing_fill-lr)*sizeof(int));
+		(os->lacing_fill-lr)*sizeof(*os->lacing_vals));
 	memmove(os->granule_vals,os->granule_vals+lr,
-		(os->lacing_fill-lr)*sizeof(ogg_int64_t));
+		(os->lacing_fill-lr)*sizeof(*os->granule_vals));
       }
       os->lacing_fill-=lr;
       os->lacing_packet-=lr;
@@ -766,6 +822,12 @@ int ogg_stream_reset(ogg_stream_state *os){
   return(0);
 }
 
+int ogg_stream_reset_serialno(ogg_stream_state *os,int serialno){
+  ogg_stream_reset(os);
+  os->serialno=serialno;
+  return(0);
+}
+
 static int _packetout(ogg_stream_state *os,ogg_packet *op,int adv){
 
   /* The last part of decode. We have the stream broken into packet
@@ -829,8 +891,8 @@ int ogg_stream_packetpeek(ogg_stream_state *os,ogg_packet *op){
 }
 
 void ogg_packet_clear(ogg_packet *op) {
-  free(op->packet);
-  memset(op, 0, sizeof(ogg_packet));
+  _ogg_free(op->packet);
+  memset(op, 0, sizeof(*op));
 }
 
 #ifdef _V_SELFTEST
@@ -1210,7 +1272,7 @@ void test_pack(const int *pl, const int **headers){
 	      ogg_stream_packetout(&os_de,&op_de); /* just catching them all */
 	      
 	      /* verify peek and out match */
-	      if(memcmp(&op_de,&op_de2,sizeof(ogg_packet))){
+	      if(memcmp(&op_de,&op_de2,sizeof(op_de))){
 		fprintf(stderr,"packetout != packetpeek! pos=%ld\n",
 			depacket);
 		exit(1);
