@@ -7,11 +7,11 @@
  *                                                                  *
  * THE OggVorbis SOURCE CODE IS (C) COPYRIGHT 1994-2001             *
  * by the XIPHOPHORUS Company http://www.xiph.org/                  *
-
+ *                                                                  *
  ********************************************************************
 
  function: utility functions for loading .vqh and .vqd files
- last mod: $Id: bookutil.c,v 1.1.1.1 2002/06/21 12:35:23 myrina Exp $
+ last mod: $Id: bookutil.c,v 1.1 2002/07/22 12:33:54 myrina Exp $
 
  ********************************************************************/
 
@@ -191,7 +191,7 @@ codebook *codebook_load(char *filename){
   }
 
   /* find the codebook struct */
-  find_seek_to(in,"static static_codebook _");
+  find_seek_to(in,"static static_codebook ");
 
   /* get the major important values */
   line=get_line(in);
@@ -534,25 +534,6 @@ void write_codebook(FILE *out,char *name,const static_codebook *c){
   int i,j,k;
 
   /* save the book in C header form */
-  fprintf(out,
-  "/********************************************************************\n"
-  " *                                                                  *\n"
-  " * THIS FILE IS PART OF THE OggVorbis SOFTWARE CODEC SOURCE CODE.   *\n"
-  " * USE, DISTRIBUTION AND REPRODUCTION OF THIS SOURCE IS GOVERNED BY *\n"
-  " * THE GNU LESSER/LIBRARY PUBLIC LICENSE, WHICH IS INCLUDED WITH    *\n"
-  " * THIS SOURCE. PLEASE READ THESE TERMS BEFORE DISTRIBUTING.        *\n"
-  " *                                                                  *\n"
-  " * THE OggVorbis SOURCE CODE IS (C) COPYRIGHT 1994-2001             *\n"
-  " * by the XIPHOPHORUS Company, http://www.xiph.org/                 *\n"
-  " *                                                                  *\n"
-  " ********************************************************************\n"
-  "\n"
-  " function: static codebook autogenerated by vq/somethingorother\n"
-  "\n"
- " ********************************************************************/\n\n");
-
-  fprintf(out,"#ifndef _V_%s_VQH_\n#define _V_%s_VQH_\n",name,name);
-  fprintf(out,"#include \"codebook.h\"\n\n");
 
   /* first, the static vectors, then the book structure to tie it together. */
   /* quantlist */
@@ -708,7 +689,7 @@ void write_codebook(FILE *out,char *name,const static_codebook *c){
 
   /* tie it all together */
   
-  fprintf(out,"static static_codebook _vq_book_%s = {\n",name);
+  fprintf(out,"static static_codebook %s = {\n",name);
   
   fprintf(out,"\t%ld, %ld,\n",c->dim,c->entries);
   fprintf(out,"\t_vq_lengthlist_%s,\n",name);
@@ -733,6 +714,4 @@ void write_codebook(FILE *out,char *name,const static_codebook *c){
     fprintf(out,"\tNULL,\n");
 
   fprintf(out,"\t0\n};\n\n");
-
-  fprintf(out,"\n#endif\n");
 }
